# Copyright 2014 NYBX Inc
# See LICENSE file for details.

"""
:module: ledgerx.settings
:synopsis: A module that lets clients specify settings.
"""

import os
import logging
import multiprocessing

from ledgerx.store import MessageIDStore
from ledgerx.utils import logger

# Core configurations

THREAD_COUNT = multiprocessing.cpu_count()
SOCKET_LINGER = 1000 # ms
ID_STORE = MessageIDStore()
LEDGERX_TEST = os.getenv('LEDGERX_TEST', False) == 'true'
DEBUG = os.getenv('DEBUG', False)

if DEBUG:
    logger.addHandler(logging.StreamHandler())

# Endpoints

def __endpoint(scheme, port):
    import socket
    try:
        if LEDGERX_TEST:
            host = '127.0.0.1'
        else:
            host = socket.gethostbyname('exchange.ledgerx.com')
    except socket.gaierror:
        host = '127.0.0.1'
        logger.critical(
                "error resolving exchange.ledgerx.com, defaulting to localhost")
    return "{0}://{1}:{2}".format(scheme, host, port)

EXCHANGE_ENDPOINT = __endpoint('tcp', 54494)
EXCHANGE_PUBLISHER_ENDPOINT = __endpoint('tcp', 54495)

