# Copyright 2014 NYBX Inc.
# All rights reserved.

"""
:module: ledgerx.test.test_commands_0_0_1
:synopsis: Unit tests for client protocol commands.
:author: Amr Ali <amr@ledgerx.com>

This test module is only concerned with v0.0.1 of protocol commands.
See <test_commands> module for version-agnostic utilities.
"""

import unittest

from ledgerx.api import client as api
from ledgerx.context import Context
from ledgerx.processor import MessageProcessor
from ledgerx.store import IDStoreItem
from ledgerx.commands.v0_0_1 import Commands
from ledgerx.test.common.context import generate_context_test_mock

class _MP(MessageProcessor):
    def handle_action_report(self, msg, prev_msg): pass
    def handle_contract_detail(self, msg, prev_msg): pass
    def handle_status_message(self, msg, prev_msg): pass
    def handle_book_top(self, msg, prev_msg): pass
    def handle_book_state(self, msg, prev_msg): pass

class TestCommands(generate_context_test_mock(_MP)):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.cmds = Commands(cls.context)

    def test_limit_order(self):
        self.__assert_command(self.cmds.limit_order,
                api.v0_0_1.MessageLimitOrder, size=1, price_in_cents=1)

    def test_market_order(self):
        self.__assert_command(self.cmds.market_order,
                api.v0_0_1.MessageMarketOrder, size=1, price_in_cents=1)

    def test_quote(self):
        self.__assert_command(self.cmds.quote,
                api.v0_0_1.MessageQuote, size=1, price_in_cents=1)

    def test_cancel_entry(self):
        self.__assert_command(self.cmds.cancel_entry,
                api.v0_0_1.MessageCancel, contract_id=0, entry_id='0')

    def test_cancel_replace_entry(self):
        self.__assert_command(self.cmds.cancel_replace_entry,
                api.v0_0_1.MessageCancelReplace, entry_id=b'0', size=1,
                price_in_cents=1)

    def test_reduce_entry_size(self):
        self.__assert_command(self.cmds.reduce_entry_size,
                api.v0_0_1.MessageReduceSize, contract_id=0, entry_id=b'0',
                size_decrement=1)

    def test_get_contract(self):
        self.__assert_command(self.cmds.get_contract,
                api.v0_0_1.MessageGetContractDetail, contract_id=0)

    def test_get_book_state(self):
        self.__assert_command(self.cmds.get_book_state,
                api.v0_0_1.MessageGetBookState, contract_id=0)

    def test_replay(self):
        self.__assert_command(self.cmds.replay,
                api.v0_0_1.MessageReplay, begin=0, end=0)

    def __assert_command(self, cmd, mtype, **kwargs):
        self.assertIsInstance(cmd(**kwargs), IDStoreItem)
        sid, msg = self.server_socket.recv_multipart()
        msg = api.protocol.MessageParser.parse(msg)
        self.assertTrue(isinstance(msg, mtype))

