# Copyright 2014 NYBX Inc.
# See LICENSE file for details.

"""
:module: ledgerx.test.test_context
:synopsis: Unit tests for the context module.
:author: Amr Ali <amr@ledgerx.com>
"""

import zmq
import time
import unittest

from threading import Event

from ledgerx.api import client as api
from ledgerx.context import Context
from ledgerx.store import IDStoreItem
from ledgerx.processor import MessageProcessor
from ledgerx.protocol.sockets import dealer_socket
from ledgerx.test.common.context import generate_context_test_mock

class _MP(MessageProcessor):
    interrupt = Event()

    def handle_action_report(self, msg, prev_msg):
        self.interrupt.set()
    def handle_contract_detail(self, msg, prev_msg):
        self.interrupt.set()
    def handle_status_message(self, msg, prev_msg):
        self.interrupt.set()
    def handle_book_top(self, msg, prev_msg):
        self.interrupt.set()
    def handle_book_state(self, msg, prev_msg):
        self.interrupt.set()

class TestContext(generate_context_test_mock(_MP)):

    def test_cmd_socket(self):
        self.cmd_socket.send_pyobj('do NOT exit')
        time.sleep(.1)
        self.assertTrue(self.context.is_alive())

    def test_context_server(self):
        msg = api.v0_0_1.MessageBookTop()
        msg.ticks = 0
        msg.timestamp = 0
        item = self.context.submit(msg)
        self.assertIsInstance(item, IDStoreItem)
        def _cb(x, y):
            self.assertEqual(x.ask, 1)
            self.assertEqual(x.bid, 2)
            self.assertEqual(y.ask, y.bid)
        item.callback = lambda x, y: _cb(x, y)

        sid, msg = self.server_socket.recv_multipart()
        msg = api.protocol.MessageParser.parse(msg)
        msg.ask = 1
        msg.bid = 2
        self.server_socket.send_multipart([sid, msg.dumps()])

        self.assertTrue(_MP.interrupt.wait(timeout=1))
        _MP.interrupt.clear()

    def test_context_publisher(self):
        msg = api.v0_0_1.MessageBookTop()
        msg.ticks = 0
        msg.timestamp = 0
        self.publisher_socket.send_multipart([b'234', msg.dumps()])

        self.assertTrue(_MP.interrupt.wait(timeout=1))
        _MP.interrupt.clear()

