# Copyright 2014 NYBX Inc.
# See LICENSE file for details.

"""
:module: ledgerx.test.test_processor
:synopsis: Unit tests for the processor module.
:author: Amr Ali <amr@ledgerx.com>
"""

import zmq
import unittest

from ledgerx.api import client as api
from ledgerx.processor import MessageProcessor

class _TestMessageProcessor(MessageProcessor):
    def handle_action_report(self, msg, prev_msg): pass
    def handle_contract_detail(self, msg, prev_msg): pass
    def handle_status_message(self, msg, prev_msg): pass
    def handle_book_top(self, msg, prev_msg): pass
    def handle_book_state(self, msg, prev_msg): pass
    def handle_event_connected(self, endpoint): return endpoint
    def handle_event_retried(self, endpoint, interval): return endpoint, interval
    def handle_event_closed(self, endpoint): return endpoint
    def handle_event_monitor_stopped(self, endpoint): return endpoint

class TestMessageProcessor(unittest.TestCase):

    def test_meta(self):
        class __TestMP(MessageProcessor): pass
        with self.assertRaises(TypeError):
            __TestMP()

        class __TestMP(MessageProcessor):
            def handle_action_report(self, msg, prev_msg): pass
        with self.assertRaises(TypeError):
            __TestMP()

        class __TestMP(MessageProcessor):
            def handle_action_report(self, msg, prev_msg): pass
            def handle_contract_detail(self, msg, prev_msg): pass
        with self.assertRaises(TypeError):
            __TestMP()

        class __TestMP(MessageProcessor):
            def handle_action_report(self, msg, prev_msg): pass
            def handle_contract_detail(self, msg, prev_msg): pass
            def handle_status_message(self, msg, prev_msg): pass
        with self.assertRaises(TypeError):
            __TestMP()

        class __TestMP(MessageProcessor):
            def handle_action_report(self, msg, prev_msg): pass
            def handle_contract_detail(self, msg, prev_msg): pass
            def handle_status_message(self, msg, prev_msg): pass
            def handle_book_top(self, msg, prev_msg): pass
        with self.assertRaises(TypeError):
            __TestMP()

        _TestMessageProcessor()

    def test_processor_handler(self):
        mp = _TestMessageProcessor()

        # Unimplemented message processor
        msg = api.protocol.ClientMessage()
        self.assertEqual(mp.handle(msg), False)

        # Unimplemented message version processor
        msg = api.protocol.ClientMessage()
        msg.mversion = '0.0.0'
        self.assertEqual(mp.handle(msg), False)

    def test_state_event_handler(self):
        mp = _TestMessageProcessor()
        self.__assert_state_event(mp, zmq.EVENT_CONNECTED)
        self.__assert_state_event(mp, zmq.EVENT_CONNECT_RETRIED, ('none', 0))
        self.__assert_state_event(mp, zmq.EVENT_CLOSED)
        self.__assert_state_event(mp, zmq.EVENT_MONITOR_STOPPED)
        self.__assert_state_event(mp, zmq.EVENT_LISTENING, True)

    def test_processor_handler_0_0_1(self):
        mp = _TestMessageProcessor()

        msg = api.v0_0_1.MessageActionReport()
        self.assertEqual(mp.handle(msg), None)

        msg = api.v0_0_1.MessageContractDetail()
        self.assertEqual(mp.handle(msg), None)

        msg = api.v0_0_1.MessageStatus()
        self.assertEqual(mp.handle(msg), None)

        msg = api.v0_0_1.MessageBookTop()
        self.assertEqual(mp.handle(msg), None)

        msg = api.v0_0_1.MessageBookState()
        self.assertEqual(mp.handle(msg), None)

    def __assert_state_event(self, mp, zmq_event, ret=None):
        state = {'event': zmq_event,
            'value': 0,
            'endpoint': 'none'}
        self.assertEqual(mp.handle_state_event(state), ret or 'none')

