# Copyright 2014 NYBX Inc.
# All rights reserved.

"""
:module: ledgerx.test.test_store
:synopsis: Unit tests for the store module.
:author: Amr Ali <amr@ledgerx.com>
"""

import unittest

from multiprocessing import Process
from ledgerx.store import MemoryStore, MessageIDStore, IDStoreItem

class TestMemoryStore(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.store = MemoryStore()
        cls.shared_store = MemoryStore(shared=True)
        cls.device = Process(target=cls.__run)
        cls.device.start()
        cls.device.join()

    def test_set(self):
        self.assertIn('remote', self.shared_store)
        self.assertNotIn('remote', self.store)
        self.assertEqual(self.shared_store['remote'], 'test value')
        with self.assertRaises(KeyError):
            self.store['remote']

    @classmethod
    def __run(cls):
        cls.store['remote'] = 'test value'
        cls.shared_store['remote'] = 'test value'

class TestIDStoreItem(unittest.TestCase):

    def test_id_store_item(self):
        item = IDStoreItem()
        self.assertIsNone(item.msg)
        self.assertTrue(callable(item.callback))

        with self.assertRaises(ValueError):
            item.callback = ''
        item.callback = lambda x, y: None
        item.msg = ''
        self.assertEqual(item.msg, '')

class TestMessageIDStore(unittest.TestCase):

    def test_store(self):
        s = MessageIDStore()
        self.assertNotIn('test', s)
        self.assertEqual(len(s), 0)

        with self.assertRaises(ValueError):
            s['test'] = 1
        item = IDStoreItem()
        s['test'] = item
        self.assertIn('test', s)
        self.assertEqual(s['test'], item)
        self.assertEqual(len(s), 1)

        del s['test']
        self.assertEqual(len(s), 0)

