# Copyright 2014 NYBX Inc.
# All rights reserved.

"""
:module: ledgerx.utils
:synopsis: Miscellaneous tools.
:author: Amr Ali <amr@ledgerx.com>
"""

import os
import sys
import logging
import importlib

from collections import deque as consume
from semantic_version import Version
from ledgerx.mixin import public

@public
def import_versions(basefile, package):
    """\
    Iterate over version files and import all version modules.

    :returns:
        A dictionary of versions `x.x.x` and imported version modules.
    """
    vers = filter(lambda x: Version.version_re.match(x.lstrip('v').replace('_', '.')),
            map(lambda x: x.rstrip('.py'),
                os.listdir(os.path.dirname(basefile))
                )
            )
    res = {}
    consume(map(lambda x: res.update(
        {x.lstrip('v').replace('_', '.'): importlib.import_module(".{0}".format(x),
            package=package)}
        ), vers), maxlen=0)
    return res

@public
def reraise(ex, message=None):
    """Reraise the exception last happened with the original traceback.

    :param ex:
        The exception to be raised instead of the original one.
    :type ex:
        A subclass of Exception.
    :param message:
        An optional message to replace the old exception's message.
    :type message:
        str
    :raises:
        ``ex``
    """
    (exc, msg, tb) = sys.exc_info()
    msg = message if message else msg
    raise ex(msg) from exc


logger = logging.getLogger('ledgerx')
logger.addHandler(logging.NullHandler())

