# Copyright 2014 NYBX Inc.
# All rights reserved.

"""
:module: ledgerx.utils.callbacks
:synopsis: Various callbacks that can be used as a callback on messages.
:author: Amr Ali <amr@ledgerx.com>
"""

from threading import Event
from ledgerx.mixin import public

@public
class ListBookState(object):
    """\
    A callback to build a full book state from a stream of book state messages.
    """

    def __init__(self):
        self._state = []
        self._is_done = Event()

    def __call__(self, msg, prev_msg):
        if not msg.is_end:
            self._state.append(msg)
        else:
            self._is_done.set()

    @property
    def state(self):
        return self._state

    @property
    def is_ready(self):
        return self._is_done.is_set()

    def wait(self, timeout=None):
        """\
        Wait until the book state is built or timeout after ``timeout`` number
        of seconds. This function always returns True, unless the function
        timed out.

        :param timeout: A floating point number of seconds. (default: None)
        :returns: Whether the function timed out.
        """
        self._is_done.wait(timeout)

