#!/usr/bin/env python
# Copyright 2014 NYBX Inc.
# See LICENSE file for details.

import os
import sys

if sys.version_info.major < 3:
    sys.stderr.write('Unsupported version of Python. Use python3.\n')
    sys.exit(1)

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
from ledgerx.version import __version__

def __filter_requires(filename):
    # Unnecessary packages for client's normal operations
    unreqs = ['sphinx', 'theme', '-e ']
    with open(filename, 'rb') as fd:
        reqs = map(lambda req: req.strip(b'\n').decode(), fd.readlines())
    for unreq in unreqs:
        reqs = filter(lambda x: unreq not in x.lower(), reqs)
    return list(reqs)

BASE_DIR = os.path.dirname(__file__)
README_PATH = os.path.join(BASE_DIR, 'README.md')
REQS_PATH = os.path.join(BASE_DIR, 'requirements.txt')

classifiers = [
    'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Natural Language :: English',
    'Intended Audience :: Developers',
    'Intended Audience :: Financial and Insurance Industry',
    'Topic :: Scientific/Engineering :: Mathematics',
    'Programming Language :: Python :: 3',
    ]

class LedgerXTest(TestCommand):

    def run_tests(self):
        os.environ['LEDGERX_TEST'] = 'true'
        TestCommand.run_tests(self)

setup(
        name = 'ledgerx-python',
        version = __version__,
        description = 'LedgerX API Python client',
        long_description = open(README_PATH).read(),
        maintainer = 'Amr Ali',
        maintainer_email = 'amr@ledgerx.com',
        url = 'http://ledgerx.com',
        install_requires = __filter_requires(REQS_PATH),
        entry_points = {
            'console_scripts': [
                'pyledgerx = ledgerx.cli:ledgerx_main',
                ],
            },
        namespace_packages = ['ledgerx'],
        packages = find_packages(exclude='test'),
        cmdclass = {'test': LedgerXTest},
        test_suite = 'ledgerx.tester.load_test_suite',
        license = 'LGPLv3+',
        platforms = 'POSIX',
        classifiers = classifiers,
    )

